@@ -22,37 +22,55 @@ package org.apache.commons.pool;
  * <p>
  * All operations defined here are essentially no-op's.
  *
+ * @param <T> the type of objects held in this pool
+ * 
  * @see PoolableObjectFactory
  * @see BaseKeyedPoolableObjectFactory
  *
  * @author Rodney Waldhoff
- * @version $Revision: 777748 $ $Date: 2009-05-22 20:00:44 -0400 (Fri, 22 May 2009) $
+ * @version $Revision: 1222388 $ $Date: 2011-12-22 13:28:27 -0500 (Thu, 22 Dec 2011) $
  * @since Pool 1.0
  */
-public abstract class BasePoolableObjectFactory implements PoolableObjectFactory {
-    public abstract Object makeObject()
-        throws Exception;
+public abstract class BasePoolableObjectFactory<T> implements PoolableObjectFactory<T> {
+    
+    /**
+     * {@inheritDoc}
+     */
+    public abstract T makeObject() throws Exception;
 
-    /** No-op. */
-    public void destroyObject(Object obj)
+    /**
+     *  No-op.
+     *  
+     *  @param obj ignored
+     */
+    public void destroyObject(T obj)
         throws Exception  {
     }
 
     /**
      * This implementation always returns <tt>true</tt>.
+     * 
+     * @param obj ignored
      * @return <tt>true</tt>
      */
-    public boolean validateObject(Object obj) {
+    public boolean validateObject(T obj) {
         return true;
     }
 
-    /** No-op. */
-    public void activateObject(Object obj)
-        throws Exception {
+    /**
+     *  No-op.
+     *  
+     *  @param obj ignored
+     */
+    public void activateObject(T obj) throws Exception {
     }
 
-    /** No-op. */
-    public void passivateObject(Object obj)
+    /**
+     *  No-op.
+     *  
+     * @param obj ignored
+     */
+    public void passivateObject(T obj)
         throws Exception {
     }
 }
