@@ -180,14 +180,16 @@ import org.apache.commons.pool.impl.GenericKeyedObjectPool.ObjectTimestampPair;
  * ensure that no call to a factory method will occur within a synchronization
  * block. See POOL-125 and DBCP-44 for more information.
  *
+ * @param <T> the type of objects held in this pool
+ * 
  * @see GenericKeyedObjectPool
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
  * @author Sandy McArthur
- * @version $Revision: 1086190 $ $Date: 2011-03-28 04:22:08 -0700 (Mon, 28 Mar 2011) $
+ * @version $Revision: 1222396 $ $Date: 2011-12-22 14:02:25 -0500 (Thu, 22 Dec 2011) $
  * @since Pool 1.0
  */
-public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
+public class GenericObjectPool<T> extends BaseObjectPool<T> implements ObjectPool<T> {
 
     //--- public constants -------------------------------------------
 
@@ -348,7 +350,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * Create a new <tt>GenericObjectPool</tt> using the specified factory.
      * @param factory the (possibly <tt>null</tt>)PoolableObjectFactory to use to create, validate and destroy objects
      */
-    public GenericObjectPool(PoolableObjectFactory factory) {
+    public GenericObjectPool(PoolableObjectFactory<T> factory) {
         this(factory, DEFAULT_MAX_ACTIVE, DEFAULT_WHEN_EXHAUSTED_ACTION, DEFAULT_MAX_WAIT, DEFAULT_MAX_IDLE,
                 DEFAULT_MIN_IDLE, DEFAULT_TEST_ON_BORROW, DEFAULT_TEST_ON_RETURN, DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS,
                 DEFAULT_NUM_TESTS_PER_EVICTION_RUN, DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS, DEFAULT_TEST_WHILE_IDLE);
@@ -359,7 +361,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param factory the (possibly <tt>null</tt>)PoolableObjectFactory to use to create, validate and destroy objects
      * @param config a non-<tt>null</tt> {@link GenericObjectPool.Config} describing my configuration
      */
-    public GenericObjectPool(PoolableObjectFactory factory, GenericObjectPool.Config config) {
+    public GenericObjectPool(PoolableObjectFactory<T> factory, GenericObjectPool.Config config) {
         this(factory, config.maxActive, config.whenExhaustedAction, config.maxWait, config.maxIdle, config.minIdle,
                 config.testOnBorrow, config.testOnReturn, config.timeBetweenEvictionRunsMillis, 
                 config.numTestsPerEvictionRun, config.minEvictableIdleTimeMillis, config.testWhileIdle, 
@@ -371,7 +373,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param factory the (possibly <tt>null</tt>)PoolableObjectFactory to use to create, validate and destroy objects
      * @param maxActive the maximum number of objects that can be borrowed from me at one time (see {@link #setMaxActive})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive) {
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive) {
         this(factory, maxActive, DEFAULT_WHEN_EXHAUSTED_ACTION, DEFAULT_MAX_WAIT, DEFAULT_MAX_IDLE, DEFAULT_MIN_IDLE,
                 DEFAULT_TEST_ON_BORROW, DEFAULT_TEST_ON_RETURN, DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS,
                 DEFAULT_NUM_TESTS_PER_EVICTION_RUN, DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS, DEFAULT_TEST_WHILE_IDLE);
@@ -385,7 +387,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param maxWait the maximum amount of time to wait for an idle object when the pool is exhausted an and
      * <i>whenExhaustedAction</i> is {@link #WHEN_EXHAUSTED_BLOCK} (otherwise ignored) (see {@link #getMaxWait})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait) {
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait) {
         this(factory, maxActive, whenExhaustedAction, maxWait, DEFAULT_MAX_IDLE, DEFAULT_MIN_IDLE, DEFAULT_TEST_ON_BORROW,
                 DEFAULT_TEST_ON_RETURN, DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS, DEFAULT_NUM_TESTS_PER_EVICTION_RUN,
                 DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS, DEFAULT_TEST_WHILE_IDLE);
@@ -403,7 +405,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param testOnReturn whether or not to validate objects after they are returned to the {@link #returnObject} method
      * (see {@link #getTestOnReturn})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             boolean testOnBorrow, boolean testOnReturn) {
         this(factory, maxActive, whenExhaustedAction, maxWait, DEFAULT_MAX_IDLE, DEFAULT_MIN_IDLE, testOnBorrow,
                 testOnReturn, DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS, DEFAULT_NUM_TESTS_PER_EVICTION_RUN,
@@ -419,7 +421,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * <i>whenExhaustedAction</i> is {@link #WHEN_EXHAUSTED_BLOCK} (otherwise ignored) (see {@link #getMaxWait})
      * @param maxIdle the maximum number of idle objects in my pool (see {@link #getMaxIdle})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait, int maxIdle) {
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait, int maxIdle) {
         this(factory, maxActive, whenExhaustedAction, maxWait, maxIdle, DEFAULT_MIN_IDLE, DEFAULT_TEST_ON_BORROW,
                 DEFAULT_TEST_ON_RETURN, DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS, DEFAULT_NUM_TESTS_PER_EVICTION_RUN,
                 DEFAULT_MIN_EVICTABLE_IDLE_TIME_MILLIS, DEFAULT_TEST_WHILE_IDLE);
@@ -438,7 +440,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param testOnReturn whether or not to validate objects after they are returned to the {@link #returnObject} method
      * (see {@link #getTestOnReturn})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             int maxIdle, boolean testOnBorrow, boolean testOnReturn) {
         this(factory, maxActive, whenExhaustedAction, maxWait, maxIdle, DEFAULT_MIN_IDLE, testOnBorrow, testOnReturn,
                 DEFAULT_TIME_BETWEEN_EVICTION_RUNS_MILLIS, DEFAULT_NUM_TESTS_PER_EVICTION_RUN,
@@ -466,7 +468,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param testWhileIdle whether or not to validate objects in the idle object eviction thread, if any
      * (see {@link #setTestWhileIdle})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             int maxIdle, boolean testOnBorrow, boolean testOnReturn, long timeBetweenEvictionRunsMillis,
             int numTestsPerEvictionRun, long minEvictableIdleTimeMillis, boolean testWhileIdle) {
         this(factory, maxActive, whenExhaustedAction, maxWait, maxIdle, DEFAULT_MIN_IDLE, testOnBorrow, testOnReturn,
@@ -495,7 +497,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param testWhileIdle whether or not to validate objects in the idle object eviction thread, if any
      *  (see {@link #setTestWhileIdle})
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             int maxIdle, int minIdle, boolean testOnBorrow, boolean testOnReturn, long timeBetweenEvictionRunsMillis,
             int numTestsPerEvictionRun, long minEvictableIdleTimeMillis, boolean testWhileIdle) {
         this(factory, maxActive, whenExhaustedAction, maxWait, maxIdle, minIdle, testOnBorrow, testOnReturn,
@@ -529,7 +531,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * (see {@link #setSoftMinEvictableIdleTimeMillis})
      * @since Pool 1.3
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             int maxIdle, int minIdle, boolean testOnBorrow, boolean testOnReturn, long timeBetweenEvictionRunsMillis,
             int numTestsPerEvictionRun, long minEvictableIdleTimeMillis, boolean testWhileIdle,
             long softMinEvictableIdleTimeMillis) {
@@ -566,7 +568,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * (see {@link #setLifo})
      * @since Pool 1.4
      */
-    public GenericObjectPool(PoolableObjectFactory factory, int maxActive, byte whenExhaustedAction, long maxWait,
+    public GenericObjectPool(PoolableObjectFactory<T> factory, int maxActive, byte whenExhaustedAction, long maxWait,
             int maxIdle, int minIdle, boolean testOnBorrow, boolean testOnReturn, long timeBetweenEvictionRunsMillis,
             int numTestsPerEvictionRun, long minEvictableIdleTimeMillis, boolean testWhileIdle,
             long softMinEvictableIdleTimeMillis, boolean lifo) {
@@ -593,7 +595,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         _softMinEvictableIdleTimeMillis = softMinEvictableIdleTimeMillis;
         _testWhileIdle = testWhileIdle;
 
-        _pool = new CursorableLinkedList();
+        _pool = new CursorableLinkedList<ObjectTimestampPair<T>>();
         startEvictor(_timeBetweenEvictionRunsMillis);
     }
 
@@ -1052,9 +1054,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @return object instance
      * @throws NoSuchElementException if an instance cannot be returned
      */
-    public Object borrowObject() throws Exception {
+    @Override
+    public T borrowObject() throws Exception {
         long starttime = System.currentTimeMillis();
-        Latch latch = new Latch();
+        Latch<T> latch = new Latch<T>();
         byte whenExhaustedAction;
         long maxWait;
         synchronized (this) {
@@ -1127,6 +1130,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                                         break;
                                     }
                                 }
+                                // see if we were awakened by a closing pool
+                                if(isClosed() == true) {
+                                    throw new IllegalStateException("Pool closed");
+                                }
                             } catch(InterruptedException e) {
                                 boolean doAllocate = false;
                                 synchronized(this) {
@@ -1178,8 +1185,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
             boolean newlyCreated = false;
             if(null == latch.getPair()) {
                 try {
-                    Object obj = _factory.makeObject();
-                    latch.setPair(new ObjectTimestampPair(obj));
+                    T obj = _factory.makeObject();
+                    latch.setPair(new ObjectTimestampPair<T>(obj));
                     newlyCreated = true;
                 } finally {
                     if (!newlyCreated) {
@@ -1244,8 +1251,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         // First use any objects in the pool to clear the queue
         for (;;) {
             if (!_pool.isEmpty() && !_allocationQueue.isEmpty()) {
-                Latch latch = (Latch) _allocationQueue.removeFirst();
-                latch.setPair((ObjectTimestampPair) _pool.removeFirst());
+                Latch<T> latch = _allocationQueue.removeFirst();
+                latch.setPair( _pool.removeFirst());
                 _numInternalProcessing++;
                 synchronized (latch) {
                     latch.notify();
@@ -1258,7 +1265,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         // Second utilise any spare capacity to create new objects
         for(;;) {
             if((!_allocationQueue.isEmpty()) && (_maxActive < 0 || (_numActive + _numInternalProcessing) < _maxActive)) {
-                Latch latch = (Latch) _allocationQueue.removeFirst();
+                Latch<T> latch = _allocationQueue.removeFirst();
                 latch.setMayCreate(true);
                 _numInternalProcessing++;
                 synchronized (latch) {
@@ -1276,7 +1283,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @throws Exception if the configured {@link PoolableObjectFactory} throws an exception destroying obj
      */
-    public void invalidateObject(Object obj) throws Exception {
+    @Override
+    public void invalidateObject(T obj) throws Exception {
         try {
             if (_factory != null) {
                 _factory.destroyObject(obj);
@@ -1304,8 +1312,9 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * while removed items are being destroyed.</li>
      * <li>Exceptions encountered destroying idle instances are swallowed.</li></ul></p>
      */
+    @Override
     public void clear() {
-        List toDestroy = new ArrayList();
+        List<ObjectTimestampPair<T>> toDestroy = new ArrayList<ObjectTimestampPair<T>>();
 
         synchronized(this) {
             toDestroy.addAll(_pool);
@@ -1324,10 +1333,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param c Collection of objects to destroy
      * @param factory PoolableConnectionFactory used to destroy the objects
      */
-    private void destroy(Collection c, PoolableObjectFactory factory) {
-        for (Iterator it = c.iterator(); it.hasNext();) {
+    private void destroy(Collection<ObjectTimestampPair<T>> c, PoolableObjectFactory<T> factory) {
+        for (Iterator<ObjectTimestampPair<T>> it = c.iterator(); it.hasNext();) {
             try {
-                factory.destroyObject(((ObjectTimestampPair)(it.next())).value);
+                factory.destroyObject(it.next().value);
             } catch(Exception e) {
                 // ignore error, keep destroying the rest
             } finally {
@@ -1344,6 +1353,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @return the number of instances currently borrowed from this pool
      */
+    @Override
     public synchronized int getNumActive() {
         return _numActive;
     }
@@ -1353,6 +1363,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @return the number of instances currently idle in this pool
      */
+    @Override
     public synchronized int getNumIdle() {
         return _pool.size();
     }
@@ -1376,7 +1387,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @param obj instance to return to the pool
      */
-    public void returnObject(Object obj) throws Exception {
+    @Override
+    public void returnObject(T obj) throws Exception {
         try {
             addObjectToPool(obj, true);
         } catch (Exception e) {
@@ -1410,7 +1422,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @param decrementNumActive whether or not to decrement the active count
      * @throws Exception
      */
-    private void addObjectToPool(Object obj, boolean decrementNumActive) throws Exception {
+    private void addObjectToPool(T obj, boolean decrementNumActive) throws Exception {
         boolean success = true;
         if(_testOnReturn && !(_factory.validateObject(obj))) {
             success = false;
@@ -1433,9 +1445,9 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                     // borrowObject always takes the first element from the queue,
                     // so for LIFO, push on top, FIFO add to end
                     if (_lifo) {
-                        _pool.addFirst(new ObjectTimestampPair(obj));
+                        _pool.addFirst(new ObjectTimestampPair<T>(obj));
                     } else {
-                        _pool.addLast(new ObjectTimestampPair(obj));
+                        _pool.addLast(new ObjectTimestampPair<T>(obj));
                     }
                     if (decrementNumActive) {
                         _numActive--;
@@ -1476,11 +1488,21 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @throws Exception
      */
+    @Override
     public void close() throws Exception {
         super.close();
         synchronized (this) {
             clear();
             startEvictor(-1L);
+
+            while(_allocationQueue.size() > 0) {
+                Latch<T> l = _allocationQueue.removeFirst();
+                
+                synchronized (l) {
+                    // notify the waiting thread
+                    l.notify();
+                }
+            }
         }
     }
 
@@ -1496,9 +1518,11 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @throws IllegalStateException when the factory cannot be set at this time
      * @deprecated to be removed in version 2.0
      */
-    public void setFactory(PoolableObjectFactory factory) throws IllegalStateException {
-        List toDestroy = new ArrayList();
-        final PoolableObjectFactory oldFactory = _factory;
+    @Deprecated
+    @Override
+    public void setFactory(PoolableObjectFactory<T> factory) throws IllegalStateException {
+        List<ObjectTimestampPair<T>> toDestroy = new ArrayList<ObjectTimestampPair<T>>();
+        final PoolableObjectFactory<T> oldFactory = _factory;
         synchronized (this) {
             assertOpen();
             if(0 < getNumActive()) {
@@ -1533,12 +1557,12 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                 return;
             }
             if (null == _evictionCursor) {
-                _evictionCursor = (_pool.cursor(_lifo ? _pool.size() : 0));
+                _evictionCursor = _pool.cursor(_lifo ? _pool.size() : 0);
             }
         }
 
         for (int i=0,m=getNumTests();i<m;i++) {
-            final ObjectTimestampPair pair;
+            final ObjectTimestampPair<T> pair;
             synchronized (this) {
                 if ((_lifo && !_evictionCursor.hasPrevious()) ||
                         !_lifo && !_evictionCursor.hasNext()) {
@@ -1547,8 +1571,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                 }
 
                 pair = _lifo ?
-                        (ObjectTimestampPair) _evictionCursor.previous() :
-                        (ObjectTimestampPair) _evictionCursor.next();
+                         _evictionCursor.previous() :
+                         _evictionCursor.next();
 
                 _evictionCursor.remove();
                 _numInternalProcessing++;
@@ -1658,12 +1682,13 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * Create an object, and place it into the pool.
      * addObject() is useful for "pre-loading" a pool with idle objects.
      */
+    @Override
     public void addObject() throws Exception {
         assertOpen();
         if (_factory == null) {
             throw new IllegalStateException("Cannot add objects without a factory.");
         }
-        Object obj = _factory.makeObject();
+        T obj = _factory.makeObject();
         try {
             assertOpen();
             addObjectToPool(obj, false);
@@ -1708,10 +1733,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         buf.append("Active: ").append(getNumActive()).append("\n");
         buf.append("Idle: ").append(getNumIdle()).append("\n");
         buf.append("Idle Objects:\n");
-        Iterator it = _pool.iterator();
+        Iterator<ObjectTimestampPair<T>> it = _pool.iterator();
         long time = System.currentTimeMillis();
         while(it.hasNext()) {
-            ObjectTimestampPair pair = (ObjectTimestampPair)(it.next());
+            ObjectTimestampPair<T> pair = it.next();
             buf.append("\t").append(pair.value).append("\t").append(time - pair.tstamp).append("\n");
         }
         return buf.toString();
@@ -1744,6 +1769,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
          * Run pool maintenance.  Evict objects qualifying for eviction and then
          * invoke {@link GenericObjectPool#ensureMinIdle()}.
          */
+        @Override
         public void run() {
             try {
                 evict();
@@ -1831,10 +1857,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * fairness. That is, objects are allocated to threads in the order that
      * threads request objects.
      */
-    private static final class Latch {
+    private static final class Latch<T> {
         
         /** object timestamp pair allocated to this latch */
-        private ObjectTimestampPair _pair;
+        private ObjectTimestampPair<T> _pair;
         
         /** Whether or not this latch may create an object instance */
         private boolean _mayCreate = false;
@@ -1843,7 +1869,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
          * Returns ObjectTimestampPair allocated to this latch
          * @return ObjectTimestampPair allocated to this latch
          */
-        private synchronized ObjectTimestampPair getPair() {
+        private synchronized ObjectTimestampPair<T> getPair() {
             return _pair;
         }
         
@@ -1851,7 +1877,7 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
          * Sets ObjectTimestampPair on this latch
          * @param pair ObjectTimestampPair allocated to this latch
          */
-        private synchronized void setPair(ObjectTimestampPair pair) {
+        private synchronized void setPair(ObjectTimestampPair<T> pair) {
             _pair = pair;
         }
 
@@ -2031,13 +2057,13 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
     private boolean _lifo = DEFAULT_LIFO;
 
     /** My pool. */
-    private CursorableLinkedList _pool = null;
+    private CursorableLinkedList<ObjectTimestampPair<T>> _pool = null;
 
     /** Eviction cursor - keeps track of idle object evictor position */
-    private CursorableLinkedList.Cursor _evictionCursor = null;
+    private CursorableLinkedList<ObjectTimestampPair<T>>.Cursor _evictionCursor = null;
 
     /** My {@link PoolableObjectFactory}. */
-    private PoolableObjectFactory _factory = null;
+    private PoolableObjectFactory<T> _factory = null;
 
     /**
      * The number of objects {@link #borrowObject} borrowed
@@ -2062,6 +2088,6 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * that objects can be allocated in the order in which the threads requested
      * them.
      */
-    private final LinkedList _allocationQueue = new LinkedList();
+    private final LinkedList<Latch<T>> _allocationQueue = new LinkedList<Latch<T>>();
 
 }
