@@ -38,13 +38,15 @@ import org.apache.commons.pool.PoolableObjectFactory;
  * re-using <tt>Object</tt>s without introducing
  * artificial limits.
  *
+ * @param <T> the type of objects held in this pool
+ * 
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
  * @author Sandy McArthur
- * @version $Revision: 965338 $ $Date: 2010-07-18 18:09:29 -0700 (Sun, 18 Jul 2010) $
+ * @version $Revision: 1222396 $ $Date: 2011-12-22 14:02:25 -0500 (Thu, 22 Dec 2011) $
  * @since Pool 1.0
  */
-public class StackObjectPool extends BaseObjectPool implements ObjectPool {
+public class StackObjectPool<T> extends BaseObjectPool<T> implements ObjectPool<T> {
     /**
      * Create a new pool using no factory. Clients must first 
      * {@link #setFactory(PoolableObjectFactory) set the factory} or
@@ -55,8 +57,9 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @see #StackObjectPool(PoolableObjectFactory)
      * @deprecated to be removed in pool 2.0 - use {@link #StackObjectPool(PoolableObjectFactory)}
      */
+    @Deprecated
     public StackObjectPool() {
-        this((PoolableObjectFactory)null,DEFAULT_MAX_SLEEPING,DEFAULT_INIT_SLEEPING_CAPACITY);
+        this(null,DEFAULT_MAX_SLEEPING,DEFAULT_INIT_SLEEPING_CAPACITY);
     }
 
     /**
@@ -70,8 +73,9 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @see #StackObjectPool(PoolableObjectFactory, int)
      * @deprecated to be removed in pool 2.0 - use {@link #StackObjectPool(PoolableObjectFactory, int)}
      */
+    @Deprecated
     public StackObjectPool(int maxIdle) {
-        this((PoolableObjectFactory)null,maxIdle,DEFAULT_INIT_SLEEPING_CAPACITY);
+        this(null,maxIdle,DEFAULT_INIT_SLEEPING_CAPACITY);
     }
 
     /**
@@ -87,8 +91,9 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @see #StackObjectPool(PoolableObjectFactory, int, int)
      * @deprecated to be removed in pool 2.0 - use {@link #StackObjectPool(PoolableObjectFactory, int, int)}
      */
+    @Deprecated
     public StackObjectPool(int maxIdle, int initIdleCapacity) {
-        this((PoolableObjectFactory)null,maxIdle,initIdleCapacity);
+        this(null,maxIdle,initIdleCapacity);
     }
 
     /**
@@ -96,7 +101,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @param factory the {@link PoolableObjectFactory} used to populate the pool
      */
-    public StackObjectPool(PoolableObjectFactory factory) {
+    public StackObjectPool(PoolableObjectFactory<T> factory) {
         this(factory,DEFAULT_MAX_SLEEPING,DEFAULT_INIT_SLEEPING_CAPACITY);
     }
 
@@ -107,7 +112,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @param factory the {@link PoolableObjectFactory} used to populate the pool
      * @param maxIdle cap on the number of "sleeping" instances in the pool
      */
-    public StackObjectPool(PoolableObjectFactory factory, int maxIdle) {
+    public StackObjectPool(PoolableObjectFactory<T> factory, int maxIdle) {
         this(factory,maxIdle,DEFAULT_INIT_SLEEPING_CAPACITY);
     }
 
@@ -126,11 +131,11 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @param initIdleCapacity initial size of the pool (this specifies the size of the container,
      *             it does not cause the pool to be pre-populated.)
      */
-    public StackObjectPool(PoolableObjectFactory factory, int maxIdle, int initIdleCapacity) {
+    public StackObjectPool(PoolableObjectFactory<T> factory, int maxIdle, int initIdleCapacity) {
         _factory = factory;
         _maxSleeping = (maxIdle < 0 ? DEFAULT_MAX_SLEEPING : maxIdle);
         int initcapacity = (initIdleCapacity < 1 ? DEFAULT_INIT_SLEEPING_CAPACITY : initIdleCapacity);
-        _pool = new Stack();
+        _pool = new Stack<T>();
         _pool.ensureCapacity( initcapacity > _maxSleeping ? _maxSleeping : initcapacity);
     }
 
@@ -156,9 +161,10 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @return an instance from the pool
      */
-    public synchronized Object borrowObject() throws Exception {
+    @Override
+    public synchronized T borrowObject() throws Exception {
         assertOpen();
-        Object obj = null;
+        T obj = null;
         boolean newlyCreated = false;
         while (null == obj) {
             if (!_pool.empty()) {
@@ -218,7 +224,8 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @param obj instance to return to the pool
      */
-    public synchronized void returnObject(Object obj) throws Exception {
+    @Override
+    public synchronized void returnObject(T obj) throws Exception {
         boolean success = !isClosed();
         if(null != _factory) {
             if(!_factory.validateObject(obj)) {
@@ -236,7 +243,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
 
         _numActive--;
         if (success) {
-            Object toBeDestroyed = null;
+            T toBeDestroyed = null;
             if(_pool.size() >= _maxSleeping) {
                 shouldDestroy = true;
                 toBeDestroyed = _pool.remove(0); // remove the stalest object
@@ -258,7 +265,8 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
     /**
      * {@inheritDoc}
      */
-    public synchronized void invalidateObject(Object obj) throws Exception {
+    @Override
+    public synchronized void invalidateObject(T obj) throws Exception {
         _numActive--;
         if (null != _factory) {
             _factory.destroyObject(obj);
@@ -272,6 +280,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @return the number of instances currently idle in this pool
      */
+    @Override
     public synchronized int getNumIdle() {
         return _pool.size();
     }
@@ -281,6 +290,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @return the number of instances currently borrowed from this pool
      */
+    @Override
     public synchronized int getNumActive() {
         return _numActive;
     }
@@ -289,9 +299,10 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * Clears any objects sitting idle in the pool. Silently swallows any
      * exceptions thrown by {@link PoolableObjectFactory#destroyObject(Object)}.
      */
+    @Override
     public synchronized void clear() {
         if(null != _factory) {
-            Iterator it = _pool.iterator();
+            Iterator<T> it = _pool.iterator();
             while(it.hasNext()) {
                 try {
                     _factory.destroyObject(it.next());
@@ -313,6 +324,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      *
      * @throws Exception never - exceptions clearing the pool are swallowed
      */
+    @Override
     public void close() throws Exception {
         super.close();
         clear();
@@ -336,12 +348,13 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * 
      * @throws Exception when the {@link #getFactory() factory} has a problem creating or passivating an object.
      */
+    @Override
     public synchronized void addObject() throws Exception {
         assertOpen();
         if (_factory == null) {
             throw new IllegalStateException("Cannot add objects without a factory.");
         }
-        Object obj = _factory.makeObject();
+        T obj = _factory.makeObject();
 
         boolean success = true;
         if(!_factory.validateObject(obj)) {
@@ -353,7 +366,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
         boolean shouldDestroy = !success;
 
         if (success) {
-            Object toBeDestroyed = null;
+            T toBeDestroyed = null;
             if(_pool.size() >= _maxSleeping) {
                 shouldDestroy = true;
                 toBeDestroyed = _pool.remove(0); // remove the stalest object
@@ -382,7 +395,9 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @throws IllegalStateException when the factory cannot be set at this time
      * @deprecated to be removed in pool 2.0
      */
-    public synchronized void setFactory(PoolableObjectFactory factory) throws IllegalStateException {
+    @Deprecated
+    @Override
+    public synchronized void setFactory(PoolableObjectFactory<T> factory) throws IllegalStateException {
         assertOpen();
         if(0 < getNumActive()) {
             throw new IllegalStateException("Objects are already active");
@@ -408,24 +423,28 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * My pool.
      * @deprecated to be made private in pool 2.0 
      */
-    protected Stack _pool = null;
+    @Deprecated
+    protected Stack<T> _pool = null;
 
     /** 
      * My {@link PoolableObjectFactory}.
      * @deprecated to be made private in pool 2.0 - use {@link #getFactory()}
      */
-    protected PoolableObjectFactory _factory = null;
+    @Deprecated
+    protected PoolableObjectFactory<T> _factory = null;
 
     /** 
      * The cap on the number of "sleeping" instances in the pool. 
      * @deprecated to be made private in pool 2.0 - use {@link #getMaxSleeping()}
      */
+    @Deprecated
     protected int _maxSleeping = DEFAULT_MAX_SLEEPING;
     
     /**
      * Number of objects borrowed but not yet returned to the pool.
      * @deprecated to be made private in pool 2.0 - use {@link #getNumActive()}
      */
+    @Deprecated
     protected int _numActive = 0;
 
     /**
@@ -434,7 +453,7 @@ public class StackObjectPool extends BaseObjectPool implements ObjectPool {
      * @return the factory
      * @since 1.5.5
      */
-    public synchronized PoolableObjectFactory getFactory() {
+    public synchronized PoolableObjectFactory<T> getFactory() {
         return _factory;
     }
 
