@@ -17,19 +17,19 @@
 
 package org.apache.commons.pool;
 
-import java.util.List;
-import java.util.Collections;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
 
 /**
  * Holds method names, parameters, and return values for tracing method calls.
  *
  * @author Sandy McArthur
- * @version $Revision: 480413 $ $Date: 2006-11-28 22:16:05 -0700 (Tue, 28 Nov 2006) $
+ * @version $Revision: 1222710 $ $Date: 2011-12-23 10:58:12 -0500 (Fri, 23 Dec 2011) $
  */
 public class MethodCall {
     private final String name;
-    private final List params;
+    private final List<?> params;
     private Object returned;
 
     public MethodCall(final String name) {
@@ -44,7 +44,7 @@ public class MethodCall {
         this(name, Arrays.asList(new Object[] {param1, param2}));
     }
 
-    public MethodCall(final String name, final List params) {
+    public MethodCall(final String name, final List<?> params) {
         if (name == null) {
             throw new IllegalArgumentException("name must not be null.");
         }
@@ -60,7 +60,7 @@ public class MethodCall {
         return name;
     }
 
-    public List getParams() {
+    public List<?> getParams() {
         return params;
     }
 
@@ -77,6 +77,7 @@ public class MethodCall {
         return this;
     }
 
+    @Override
     public boolean equals(final Object o) {
         if (this == o) return true;
         if (o == null || getClass() != o.getClass()) return false;
@@ -90,6 +91,7 @@ public class MethodCall {
         return true;
     }
 
+    @Override
     public int hashCode() {
         int result;
         result = (name != null ? name.hashCode() : 0);
@@ -98,6 +100,7 @@ public class MethodCall {
         return result;
     }
 
+    @Override
     public String toString() {
         final StringBuffer sb = new StringBuffer();
         sb.append("MethodCall");
