@@ -20,7 +20,7 @@ import junit.framework.TestCase;
 
 /**
  * @author Rodney Waldhoff
- * @version $Revision: 901944 $ $Date: 2010-01-21 17:27:04 -0700 (Thu, 21 Jan 2010) $
+ * @version $Revision: 1222396 $ $Date: 2011-12-22 14:02:25 -0500 (Thu, 22 Dec 2011) $
  */
 public class TestBaseKeyedPoolableObjectFactory extends TestCase {
     public TestBaseKeyedPoolableObjectFactory(String testName) {
@@ -28,8 +28,9 @@ public class TestBaseKeyedPoolableObjectFactory extends TestCase {
     }
 
     public void testDefaultMethods() throws Exception {
-        KeyedPoolableObjectFactory factory = new BaseKeyedPoolableObjectFactory() { 
-            public Object makeObject(Object key) throws Exception {
+        KeyedPoolableObjectFactory<String, Object> factory = new BaseKeyedPoolableObjectFactory<String, Object>() { 
+            @Override
+            public Object makeObject(String key) throws Exception {
                 return null;
             }
         };   
