@@ -26,36 +26,37 @@ import org.apache.commons.pool.TestObjectPoolFactory;
  * Tests for {@link StackObjectPoolFactory}.
  *
  * @author Sandy McArthur
- * @version $Revision: 901944 $ $Date: 2010-01-21 17:27:04 -0700 (Thu, 21 Jan 2010) $
+ * @version $Revision: 1222396 $ $Date: 2011-12-22 14:02:25 -0500 (Thu, 22 Dec 2011) $
  */
 public class TestStackObjectPoolFactory extends TestObjectPoolFactory {
     public TestStackObjectPoolFactory(final String name) {
         super(name);
     }
 
-    protected ObjectPoolFactory makeFactory(final PoolableObjectFactory objectFactory) throws UnsupportedOperationException {
-        return new StackObjectPoolFactory(objectFactory);
+    @Override
+    protected ObjectPoolFactory<Integer> makeFactory(final PoolableObjectFactory<Integer> objectFactory) throws UnsupportedOperationException {
+        return new StackObjectPoolFactory<Integer>(objectFactory);
     }
 
     public void testConstructors() throws Exception {
-        StackObjectPoolFactory factory = new StackObjectPoolFactory();
+        StackObjectPoolFactory<Integer> factory = new StackObjectPoolFactory<Integer>();
         factory.createPool().close();
 
         
-        factory = new StackObjectPoolFactory(1);
-        StackObjectPool pool = (StackObjectPool)factory.createPool();
+        factory = new StackObjectPoolFactory<Integer>(1);
+        StackObjectPool<Integer> pool = (StackObjectPool<Integer>)factory.createPool();
         pool.close();
 
 
-        factory = new StackObjectPoolFactory(1, 1);
-        pool = (StackObjectPool)factory.createPool();
+        factory = new StackObjectPoolFactory<Integer>(1, 1);
+        pool = (StackObjectPool<Integer>)factory.createPool();
         pool.close();
 
 
-        factory = new StackObjectPoolFactory(new MethodCallPoolableObjectFactory(), 1);
-        pool = (StackObjectPool)factory.createPool();
-        Object a = pool.borrowObject();
-        Object b = pool.borrowObject();
+        factory = new StackObjectPoolFactory<Integer>(new MethodCallPoolableObjectFactory(), 1);
+        pool = (StackObjectPool<Integer>)factory.createPool();
+        Integer a = pool.borrowObject();
+        Integer b = pool.borrowObject();
         pool.returnObject(a);
         pool.returnObject(b);
         assertEquals(1, pool.getNumIdle());
