@@ -23,13 +23,13 @@ import org.apache.commons.pool.impl.GenericObjectPool;
  * Multi-thread performance test
  * 
  * @author Dirk Verbeeck
- * @version $Revision: 775703 $ $Date: 2009-05-17 09:39:51 -0700 (Sun, 17 May 2009) $ 
+ * @version $Revision: 1221705 $ $Date: 2011-12-21 08:03:54 -0500 (Wed, 21 Dec 2011) $ 
  */
 public class PerformanceTest {
     private int logLevel = 0;
     private int nrIterations = 5;
 
-    private GenericObjectPool pool;
+    private GenericObjectPool<Integer> pool;
     private boolean start = false;
     private volatile int waiting = 0;
     private volatile int complete = 0;
@@ -62,7 +62,7 @@ public class PerformanceTest {
                     System.out.println(name + "   waiting: " + waiting + "   complete: " + complete);
                 }
                 long bbegin = System.currentTimeMillis();
-                Object o = pool.borrowObject();
+                Integer o = pool.borrowObject();
                 long bend = System.currentTimeMillis();
                 waiting--;
                 do {
@@ -111,7 +111,7 @@ public class PerformanceTest {
         
         SleepingObjectFactory factory = new SleepingObjectFactory();
         if (logLevel >= 4) { factory.setDebug(true); } 
-        pool = new GenericObjectPool(factory);
+        pool = new GenericObjectPool<Integer>(factory);
         pool.setMaxActive(maxActive);
         pool.setMaxIdle(maxIdle);
         pool.setTestOnBorrow(true);
