@@ -23,37 +23,37 @@ import org.apache.commons.pool.PoolableObjectFactory;
  * Sleepy ObjectFactory (everything takes a while longer)
  * 
  * @author Dirk Verbeeck
- * @version $Revision: 777749 $ $Date: 2009-05-22 17:04:54 -0700 (Fri, 22 May 2009) $ 
+ * @version $Revision: 1221705 $ $Date: 2011-12-21 08:03:54 -0500 (Wed, 21 Dec 2011) $ 
  */
-public class SleepingObjectFactory implements PoolableObjectFactory {
+public class SleepingObjectFactory implements PoolableObjectFactory<Integer> {
 
     private int counter = 0;
     private boolean debug = false;
 
-    public Object makeObject() throws Exception {
-        Object obj = new Integer(counter++);
+    public Integer makeObject() throws Exception {
+        Integer obj = new Integer(counter++);
         debug("makeObject", obj);
         sleep(500);
         return obj;
     }
 
-    public void destroyObject(Object obj) throws Exception {
+    public void destroyObject(Integer obj) throws Exception {
         debug("destroyObject", obj);
         sleep(250);
     }
 
-    public boolean validateObject(Object obj) {
+    public boolean validateObject(Integer obj) {
         debug("validateObject", obj);
         sleep(30);
         return true;
     }
 
-    public void activateObject(Object obj) throws Exception {
+    public void activateObject(Integer obj) throws Exception {
         debug("activateObject", obj);
         sleep(10);
     }
 
-    public void passivateObject(Object obj) throws Exception {
+    public void passivateObject(Integer obj) throws Exception {
         debug("passivateObject", obj);
         sleep(10);
     }
